@@ -17,7 +17,7 @@
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 2003 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,7 +53,6 @@
 #include "MagickCore/monitor-private.h"
 #include "MagickCore/option.h"
 #include "MagickCore/pixel-accessor.h"
-#include "MagickCore/pixel-private.h"
 #include "MagickCore/resource_.h"
 #include "MagickCore/string-private.h"
 #include "MagickCore/thread-private.h"
@@ -74,7 +73,7 @@
 %
 %  ChannelFxImage() applies a channel expression to the specified image.  The
 %  expression consists of one or more channels, either mnemonic or numeric (e.g.
-%  red, 1), separated by actions as follows:
+%  r, red, 0), separated by actions as follows:
 %
 %    <=>     exchange two channels (e.g. red<=>blue)
 %    =>      copy one channel to another channel (e.g. red=>green)
@@ -123,7 +122,7 @@ static MagickBooleanType ChannelImage(Image *destination_image,
     *destination_view;
 
   MagickBooleanType
-    status;
+    status = MagickTrue;
 
   size_t
     height,
@@ -132,11 +131,10 @@ static MagickBooleanType ChannelImage(Image *destination_image,
   ssize_t
     y;
 
-  status=MagickTrue;
-  source_view=AcquireVirtualCacheView(source_image,exception);
-  destination_view=AcquireAuthenticCacheView(destination_image,exception);
   height=MagickMin(source_image->rows,destination_image->rows);
   width=MagickMin(source_image->columns,destination_image->columns);
+  source_view=AcquireVirtualCacheView(source_image,exception);
+  destination_view=AcquireAuthenticCacheView(destination_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(static) shared(status) \
     magick_number_threads(source_image,source_image,height,1)
@@ -147,13 +145,13 @@ static MagickBooleanType ChannelImage(Image *destination_image,
       destination_traits,
       source_traits;
 
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -197,13 +195,13 @@ MagickExport Image *ChannelFxImage(const Image *image,const char *expression,
 #define ChannelFxImageTag  "ChannelFx/Image"
 
   ChannelFx
-    channel_op;
+    channel_op = ExtractChannelOp;
 
   ChannelType
     channel_mask;
 
   char
-    token[MagickPathExtent];
+    token[MagickPathExtent] = "";
 
   const char
     *p;
@@ -212,29 +210,30 @@ MagickExport Image *ChannelFxImage(const Image *image,const char *expression,
     *source_image;
 
   double
-    pixel;
+    pixel = 0.0;
 
   Image
     *destination_image;
 
   MagickBooleanType
-    status;
+    status = MagickTrue;
 
   PixelChannel
     source_channel,
-    destination_channel;
+    destination_channel = RedPixelChannel;
 
   ssize_t
-    channels;
+    channels = 0;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  p=expression;
   source_image=image;
-  destination_image=CloneImage(source_image,0,0,MagickTrue,exception);
+  destination_image=CloneImage(image,0,0,MagickTrue,exception);
   if (destination_image == (Image *) NULL)
     return((Image *) NULL);
   if (expression == (const char *) NULL)
@@ -245,13 +244,9 @@ MagickExport Image *ChannelFxImage(const Image *image,const char *expression,
       destination_image=GetLastImageInList(destination_image);
       return((Image *) NULL);
     }
-  destination_channel=RedPixelChannel;
-  channel_mask=UndefinedChannel;
-  pixel=0.0;
-  p=(char *) expression;
+  channel_mask=destination_image->channel_mask;
   (void) GetNextToken(p,&p,MagickPathExtent,token);
-  channel_op=ExtractChannelOp;
-  for (channels=0; *token != '\0'; )
+  while (*token != '\0')
   {
     ssize_t
       i;
@@ -268,10 +263,9 @@ MagickExport Image *ChannelFxImage(const Image *image,const char *expression,
       }
       case '|':
       {
+        source_image=GetFirstImageInList(source_image);
         if (GetNextImageInList(source_image) != (Image *) NULL)
           source_image=GetNextImageInList(source_image);
-        else
-          source_image=GetFirstImageInList(source_image);
         (void) GetNextToken(p,&p,MagickPathExtent,token);
         break;
       }
@@ -304,7 +298,7 @@ MagickExport Image *ChannelFxImage(const Image *image,const char *expression,
         (void) GetNextToken(p,&p,MagickPathExtent,token);
         channels=0;
         destination_channel=RedPixelChannel;
-        channel_mask=UndefinedChannel;
+        channel_mask=destination_image->channel_mask;
         break;
       }
       default:
@@ -358,10 +352,6 @@ MagickExport Image *ChannelFxImage(const Image *image,const char *expression,
               }
           }
         destination_channel=(PixelChannel) i;
-        if (i >= (ssize_t) GetPixelChannels(destination_image))
-          (void) SetPixelMetaChannels(destination_image,(size_t) (
-            destination_channel-GetPixelChannels(destination_image)+1),
-            exception);
         if (image->colorspace != UndefinedColorspace)
           switch (destination_channel)
           {
@@ -403,7 +393,8 @@ MagickExport Image *ChannelFxImage(const Image *image,const char *expression,
               break;
             }
           }
-        channel_mask=(ChannelType) (channel_mask | ParseChannelOption(token));
+        channel_mask=(ChannelType) (channel_mask |
+          (1UL << ParseChannelOption(token)));
         if (((channels >= 1)  || (destination_channel >= 1)) &&
             (IsGrayColorspace(destination_image->colorspace) != MagickFalse))
           (void) SetImageColorspace(destination_image,sRGBColorspace,exception);
@@ -513,7 +504,7 @@ MagickExport Image *CombineImages(const Image *image,
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
@@ -574,13 +565,13 @@ MagickExport Image *CombineImages(const Image *image,
     Quantum
       *pixels;
 
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       i;
 
     if (status == MagickFalse)
@@ -595,7 +586,7 @@ MagickExport Image *CombineImages(const Image *image,
     next=image;
     for (i=0; i < (ssize_t) GetPixelChannels(combine_image); i++)
     {
-      register ssize_t
+      ssize_t
         x;
 
       PixelChannel channel = GetPixelChannelChannel(combine_image,i);
@@ -613,7 +604,7 @@ MagickExport Image *CombineImages(const Image *image,
       {
         if (x < (ssize_t) next->columns)
           {
-            q[i]=GetPixelGray(next,p);
+            q[i]=GetPixelIntensity(next,p);
             p+=GetPixelChannels(next);
           }
         q+=GetPixelChannels(combine_image);
@@ -671,7 +662,7 @@ MagickExport Image *CombineImages(const Image *image,
 MagickExport MagickBooleanType GetImageAlphaChannel(const Image *image)
 {
   assert(image != (const Image *) NULL);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(image->signature == MagickCoreSignature);
   return(image->alpha_trait != UndefinedPixelTrait ? MagickTrue : MagickFalse);
@@ -732,7 +723,7 @@ MagickExport Image *SeparateImage(const Image *image,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
@@ -760,13 +751,13 @@ MagickExport Image *SeparateImage(const Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -781,7 +772,7 @@ MagickExport Image *SeparateImage(const Image *image,
       }
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      register ssize_t
+      ssize_t
         i;
 
       SetPixelChannel(separate_image,GrayPixelChannel,(Quantum) 0,q);
@@ -852,12 +843,12 @@ MagickExport Image *SeparateImages(const Image *image,ExceptionInfo *exception)
     *images,
     *separate_image;
 
-  register ssize_t
+  ssize_t
     i;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   images=NewImageList();
   for (i=0; i < (ssize_t) GetPixelChannels(image); i++)
@@ -910,15 +901,14 @@ MagickExport Image *SeparateImages(const Image *image,ExceptionInfo *exception)
 */
 
 static inline void FlattenPixelInfo(const Image *image,const PixelInfo *p,
-  const double alpha,const Quantum *q,const double beta,
-  Quantum *composite)
+  const double alpha,const Quantum *q,const double beta,Quantum *composite)
 {
   double
     Da,
     gamma,
     Sa;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -984,7 +974,7 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
     y;
 
   assert(image != (Image *) NULL);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(image->signature == MagickCoreSignature);
   status=MagickTrue;
@@ -992,6 +982,8 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
   {
     case ActivateAlphaChannel:
     {
+      if (image->alpha_trait == BlendPixelTrait)
+        return(status);
       image->alpha_trait=BlendPixelTrait;
       break;
     }
@@ -1010,10 +1002,10 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
 #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register Quantum
+        Quantum
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -1030,7 +1022,7 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
           double
             gamma;
 
-          register ssize_t
+          ssize_t
             i;
 
           gamma=QuantumScale*GetPixelAlpha(image,q);
@@ -1070,10 +1062,10 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
 #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register Quantum
+        Quantum
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -1130,10 +1122,10 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
 #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register Quantum
+        Quantum
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -1151,7 +1143,7 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
             gamma,
             Sa;
 
-          register ssize_t
+          ssize_t
             i;
 
           Sa=QuantumScale*GetPixelAlpha(image,q);
@@ -1191,6 +1183,8 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
     }
     case OffAlphaChannel:
     {
+      if (image->alpha_trait == UndefinedPixelTrait)
+        return(status);
       image->alpha_trait=UndefinedPixelTrait;
       break;
     }
@@ -1223,10 +1217,10 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
 #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register Quantum
+        Quantum
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -1281,10 +1275,10 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
         PixelInfo
           pixel;
 
-        register Quantum
+        Quantum
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -1311,39 +1305,7 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
     }
     case TransparentAlphaChannel:
     {
-      image->alpha_trait=BlendPixelTrait;
-      image_view=AcquireAuthenticCacheView(image,exception);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static) shared(status) \
-        magick_number_threads(image,image,image->rows,1)
-#endif
-      for (y=0; y < (ssize_t) image->rows; y++)
-      {
-        register Quantum
-          *magick_restrict q;
-    
-        register ssize_t
-          x;
-    
-        if (status == MagickFalse)
-          continue;
-        q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
-          exception);
-        if (q == (Quantum *) NULL)
-          {
-            status=MagickFalse;
-            continue;
-          }
-        for (x=0; x < (ssize_t) image->columns; x++)
-        {
-          if (GetPixelWriteMask(image,q) <= (QuantumRange/2))
-            SetPixelAlpha(image,TransparentAlpha,q);
-          q+=GetPixelChannels(image);
-        }
-        if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
-          status=MagickFalse;
-      }
-      image_view=DestroyCacheView(image_view);
+      status=SetImageAlpha(image,TransparentAlpha,exception);
       break;
     }
     case UndefinedAlphaChannel:
